//======================================================= IBJpluris.c
//
// Plume Rise Model PLURIS
// =======================
//
// For details see: 
// Janicke, U., Janicke, L.: A three-dimensional plume rise model for
// dry and wet plumes, Atmospheric Environment 35 (2000), 877-890.
//
// Copyright (C) Janicke Consulting, berlingen, Germany, 1998-2010
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// history:
//
// 1999-01-24 uj  2.0   implementation of IBJ routines
// 1999-05-09 uj  2.1   new entrainment function
// 2000-01-20 uj  2.2   top hat for visible radius
// 2000-05-12 uj  2.3   added mass concept etc. (used for publication)
// 2000-07-04 uj  2.4   implementation of boundary layer model IBJprf
// 2001-04-18 uj  2.4.2 update of IBJ routines
// 2002-03-23 uj  2.5.0 update, input profiles in dmna format
// 2002-03-23 uj  2.5.2 use ha for calculating us
// 2003-05-14 uj  2.5.3 eta in the initial zone corrected, small output changes
// 2006-12-27 uj  2.6.1 "QQ" changed to "LQ", "QQ" for heat current (MW)
// 2006-12-27 uj  2.6.2 final rise introduced (-f)
// 2006-12-27 uj  2.6.3 IBJdmn/ary/txt/msg from LASAT 2.14, gcc compiler
// 2010-11-10 uj  2.7.1 Temperature profile from KTA
//
//===================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <signal.h>

#define VERSION "2.7.1"

#include "IBJmsg.h"
#include "IBJprm.h"
#include "IBJtbl.h"
#include "IBJary.h"
#include "IBJdmn.h"
#include "IBJpluris.h"
#include "IBJprf.h"

char *argv[20];
double  var[20], dvds[20];
int  argc, NewProfile=0, CutOff=1, FinalRiseBreak=0;
double FinalRise = 0, Zmax = ZMAX;
char sFinal[256];
double SigC = 1.; // radius(core plume) = RR/sqrt(SigC), set to SIGC

double getRise_VDI_3782_3(double x, int km, double hq, double dq, double vq, 
double uq, double qq, double *xmax);

//----------------------------------------------------------------------- GetQs
// returns the saturation humidity for the specified temperature and pressure
// (implementation of the Goff-Gratch equations)
double GetQs(double t, double p)
{
  double qs, t1, tt1, t1t, logp;
  logp = log(p/100.);
  t1   = TZERO;
  tt1  = t/t1;
  t1t  = t1/t;
  if (t > t1)
    qs = 2.303*(10.79574*(1.-t1t)+0.000150474*(1-pow(10.,-8.2969*(tt1-1)))
                +0.00042873*(pow(10.,4.76955*(1-t1t))-1))
         - 5.028*log(tt1) - logp + 1.355;
  else
    qs = 2.303*(-9.09685*(t1t-1)+0.87682*(1-tt1))
         - 3.56654*log(t1t) - logp + 1.355;
  qs = exp(qs);
  if (qs > 1.) qs = 1.;
  return qs;
}

//---------------------------------------------------------------------- GetEnt
// returns the entrainment
double GetEnt( void )
{
  double ent, frd, dup, dup2, dus2, f;
  f    = KK(1)*KL(1) + KK(2)*KL(2) + KK(3)*KL(3);
  dup  = f/UU - UU;
  dup2 = dup*dup;
  dus2 = fabs(UL*UL - f*f/(UU*UU));
  frd  = (DL != DD) ? UU*UU*DL/(GRAV*RR*fabs(DL-DD)) : 1./MINVAL;
  ent = RR*((ALPHA1/2.*dup2 + ALPHA2*dus2)/sqrt(dup2+dus2) + ALPHA4*UU/frd);
  return EF*ent;
}

//------------------------------------------------------------------ GetDensity
// returns the density
double GetDensity(double t, double q, double sig, double p) {
  return ( p/(RAIR*t)/(1. - q - sig + (RVAPOUR/RAIR)*q) );
  }

//----------------------------------------------------------------- GetPressure
// returns the pressure for the specified height index
double GetPressure( int ih )
{
  int i;
  double p, it;
  if (ih < 2) return PA;
  it = 0.;
  for (i=1; i<ih; i++)
     it += 0.5*(ZV(i+1)-ZV(i))*(1./TV(i+1) + 1./TV(i));
  p = PA * exp(-GRAV/RAIR*it);
  return p;
}

//---------------------------------------------------------------------- SetAmb
// sets the ambient parameters for the specidied location
int SetAmb( double x, double y, double z)
{
  register int i, l, n;
  double d1, d2;
  double dudz, dtdz, dcdz, dqdz, dpdz, djdz, dkldz[4];
  for (i=1; i<NZ; i++)
     if ((z >= ZV(i)) && (z < ZV(i+1))) break;
  l = 0;
  if (z >= ZV(NZ)) l = NZ;
  if (z < ZV(1)) l = 1;
  if (l != 0) {
    QL = QV(l);
    TL = TV(l);
    UL = UV(l);
    JL = JV(l);
    KL(1) = XV(l);
    KL(2) = YV(l);
    KL(3) = WV(l);
    PL = PV(l);
    CL = CV(l);
    for (n=1; n<=3; n++) dkldz[n] = 0.;
    dtdz  = 0.;
    dudz  = 0.;
    dqdz  = 0.;
    dpdz  = 0.;
    djdz  = 0.;
    dcdz  = 0.;
  }
  else {
    d1 = ZV(i+1)-ZV(i);
    d2 = z - ZV(i);
    dtdz      = (TV(i+1) - TV(i))/d1;
    dudz      = (UV(i+1) - UV(i))/d1;
    dkldz[1]  = (XV(i+1) - XV(i))/d1;
    dkldz[2]  = (YV(i+1) - YV(i))/d1;
    dkldz[3]  = (WV(i+1) - WV(i))/d1;
    dqdz      = (QV(i+1) - QV(i))/d1;
    djdz      = (JV(i+1) - JV(i))/d1;
    dpdz      = (PV(i+1) - PV(i))/d1;
    dcdz      = (CV(i+1) - CV(i))/d1;
    TL    = TV(i) + dtdz*d2;
    UL    = UV(i) + dudz*d2;
    KL(1) = XV(i) + dkldz[1]*d2;
    KL(2) = YV(i) + dkldz[2]*d2;
    KL(3) = WV(i) + dkldz[3]*d2;
    JL    = JV(i) + djdz*d2;
    PL    = PV(i) + dpdz*d2;
    CL    = CV(i) + dcdz*d2;
    QL    = QV(i) + dqdz*d2;
  }
  ZL = QL;
  SL = 0.;
  DL = GetDensity(TL, QL, SL, PL);
  return 0;
}

//------------------------------------------------------------------- SetAmbPrf
// Initializes the ambient profiles for the vertical grid
int SetAmbPrf( void )
{
  dP(SetAmbPrf);
  int i;
  double uh;
  float u10, lm, z0;
  int kl;
  US = 0.;
  PrfSetD0(D0);
  PrfSetHa(HA);
  PrfSetRa(DA);
  if (LM != 0.) { // profiles from boundary layer model
    US = UA/PrfGetU(HA, LM, 800., 1.0, Z0, 600.);
    if (US == 0.)                                                  eX(14);
    if (HM <= 0.) 
      HM = PrfGetHM(KM, (float)LM, (float)US);
    u10 = PrfGetU(10.0, LM, 800., US, Z0, 600.);
    lm = (float)LM;
    z0 = (float)Z0;
    PrfGetStability(z0, &lm, &kl);
    LM = (double)lm;
  }
  else
    VE = -VE;
  for (i=1; i<=NZ; i++) {
    if (i<NZ) ZV(i) = (i-1)*ZA;
    else ZV(i) = ((NZ-1)*ZA>4000.) ? (NZ-1)*ZA : 4000.;
    if (US == 0.) {
      uh = (ZV(i) > 200.) ? 200. : ZV(i);
      UV(i) = (UX != 0.) ? UA * pow(uh/UH, UX) : UA;
      TV(i) = TZERO + TA + (ZV(i)-TH)*TX;
    }
    else {
      UV(i) = PrfGetU(ZV(i), LM, HM, US, Z0, 600.);
      TV(i) = PrfGetTmpKTA(ZV(i), TH, TZERO+TA, u10, kl);          //-2010-11-10
      //TV(i) = PrfGetTmp(ZV(i), LM, US, Z0, TZERO+TA, TZERO+TA);
    }
    SV(i) = PrfGetSigW(ZV(i), LM, HM, US, Z0, 600.);
    DV(i) = (LM != 0.) ? PrfGetRa(ZV(i), LM, HM, Z0) : DA;
    XV(i) = -UV(i)*sin(DV(i)*PIFAC/180.);
    YV(i) = -UV(i)*cos(DV(i)*PIFAC/180.);
    WV(i) = 0.;
    IV(i) = IA;
    if (IV(i)<0. || IV(i)>1.)                                       eX(11);
    JV(i) = IV(i)*UV(i);
    PV(i) = GetPressure(i);
    RV(i) = RA;
    if (RV(i)<0. || RV(i)>1.)                                       eX(12);
    QV(i) = RV(i) * GetQs(TV(i), PV(i));
    CV(i) = CA;
    if (CV(i) < 0.)                                                 eX(13);
  }
  return 0;
eX_11:
  eMSG("Error in profiles: iv must be between 0 and 1 (%s)\n", ePGMs);
eX_12:
  eMSG("Error in profiles: pv must be between 0 and 1 (%s)\n", ePGMs);
eX_13:
  eMSG("Error in profiles: cv must be > 0 (%s)\n", ePGMs);
eX_14:
  eMSG("Error in profiles: calculated ust = 0 (%s)\n", ePGMs);
}

//------------------------------------------------------------------ ReadAmbPrf
// reads the vertical grid and ambient profiles from the profile file
int ReadAmbPrf( void )
{
  int i, j, i1, i2, nz, ostd;
  char elnm[10];
  TXTSTR usrhdr = { NULL, 0 };
  TXTSTR syshdr = { NULL, 0 };
  ARYDSC pa;
  DMNFRMREC *pfr;
  dP(ReadAmbPrf);
  memset(&pa, 0, sizeof(ARYDSC));
  if (0 > DmnRead(PrfName, &usrhdr, &syshdr, &pa))                   eX(11);
  if (pa.elmsz != 28 || pa.numdm != 1)                               eX(12);
  ostd = 0;
  strcpy(elnm, "ha");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  ostd += sizeof(float);
  strcpy(elnm, "ua");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  ostd += sizeof(float);
  strcpy(elnm, "da");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  ostd += sizeof(float);
  strcpy(elnm, "ia");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  ostd += sizeof(float);
  strcpy(elnm, "ta");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  ostd += sizeof(float);
  strcpy(elnm, "ra");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  ostd += sizeof(float);
  strcpy(elnm, "ca");
  pfr = DmnFrmPointer(DmnFrmTab, elnm);  if (!pfr)                   eX(15);
  if (pfr->offset != ostd)                                           eX(16);
  if (pfr->dt != flDAT)                                              eX(17);
  i1 = pa.bound[0].low;
  i2 = pa.bound[0].hgh;
  nz = i2-i1+1;
  if (nz > NZ)                                                       eX(20);
  NZ = nz;
  for (i=1,j=i1; i<=NZ; i++, j++) {
    ZV(i) = ((AKTREC*)AryPtrX(&pa, j))->ha;
    if (i>1) if (ZV(i) <= ZV(i-1))                                   eX(31);
    UV(i) = ((AKTREC*)AryPtrX(&pa, j))->ua;
    DV(i) = ((AKTREC*)AryPtrX(&pa, j))->da;
    XV(i) = -UV(i)*sin(DV(i)*PIFAC/180.);
    YV(i) = -UV(i)*cos(DV(i)*PIFAC/180.);
    WV(i) = 0.;
    IV(i) = ((AKTREC*)AryPtrX(&pa, j))->ia;
    if (IV(i)<0. || IV(i)>1.)                                        eX(32);
    JV(i) = IV(i)*UV(i);
    TV(i) = TZERO + ((AKTREC*)AryPtrX(&pa, j))->ta;
    PV(i) = GetPressure(i);
    RV(i) = ((AKTREC*)AryPtrX(&pa, j))->ra;
    if (RV(i)<0. || RV(i)>1.)                                        eX(33);
    QV(i) = RV(i) * GetQs(TV(i), PV(i));
    CV(i) = ((AKTREC*)AryPtrX(&pa, j))->ca;
    if (CV(i) < 0.)                                                  eX(34);
  }
  return NZ;
eX_11:
  eMSG("Can't read profile file %s (%s)\n", PrfName, ePGMs);
eX_12:
  eMSG("Wrong data format for profiles in%s (%s)\n"
       "(example: format ha%%f ua%%f da%%f ia%%f ta%%f ra%%f ca%%f)\n", 
       PrfName, ePGMs);
eX_15:
  eMSG("No element %s found in profile file %s (%s)\n", elnm, PrfName, ePGMs);
eX_16:
  eMSG("Element %s has wrong format (%s)\n"
       "(example: format ha%%f ua%%f da%%f ia%%f ta%%f ra%%f ca%%f)\n", 
       elnm, ePGMs);
eX_17:
  eMSG("Wrong order of elements\n"
       "(example: format ha%%f ua%%f da%%f ia%%f ta%%f ra%%f ca%%f)\n", 
       PrfName, ePGMs);
eX_20:
  eMSG("Maximum number of anemometer heights %d exceeded (%s)\n", NZ, ePGMs);
eX_31:
  eMSG("Anemometer heights not in increasing order (%s)\n", ePGMs);
eX_32:
  eMSG("Ambient turbulence intensity not in the range [0,1] (%s)\n", ePGMs);
eX_33:
  eMSG("Ambient relative humidity not in the range [0,1] (%s)\n", ePGMs);
eX_34:
  eMSG("Ambient concentration is negative (%s)\n", ePGMs);
}

//------------------------------------------------------------------ Initialize
// initializes all parameters
int Initialize( void )
{
  int i;
  float z0, lm;
  dP(Initialize);
  if (HQ < 0.)                                                       eX(10);
  if (UQ < 0.)                                                       eX(11);
  if (IQ < 0.)                                                       eX(12);
  if (QQ > 0) {
    double f = 1 - QQ/(PIFAC*0.25*DQ*DQ*UQ*1.36e-3*TZERO);
    TQ = (f != 0) ? (TZERO+10.)/f - TZERO : -999;
  }
  if (TQ < 0)                                                        eX(13);
  QQ = PIFAC*0.25*DQ*DQ*UQ*(TZERO/(TZERO+TQ))*1.36e-3*(TQ-10.);
  if (DQ <= 0.)                                                      eX(14);
  if (LQ < 0. || LQ > 1.)                                            eX(15);
  if (RQ < 0. || RQ > 1.)                                            eX(16);
  if (SQ < 0. || SQ > 1.)                                            eX(17);
  if (CQ < 0.)                                                       eX(19);
  if (UA < 0.)                                                       eX(20);
  if (TA < -TZERO)                                                   eX(21);
  if (RA < 0. || RA > 1.)                                            eX(22);
  if (IA < 0.)                                                       eX(23);
  if (CA < 0.)                                                       eX(24);
  if (SC <= 0.)                                                      eX(25);
  if (SD <= 0.)                                                      eX(26);
  if (SN <= 0)                                                       eX(27);
  if (SE <= 0.)                                                      eX(28);
  if (PA <= 0.)                                                      eX(29);
  if (UH <= 0.)                                                      eX(32);
  // if (HM <= 0.)                                                      eX(33);
  if (Z0 <= 0.)                                                      eX(34);
  if ((LM != 0 && KM == 0) || (LM == 0 && KM != 0)) {
    lm = LM;
    z0 = Z0;
    PrfGetStability(z0, &lm, &KM);
    LM = lm;
  }
  PrfFile = fopen(PrfName, "r");
  if (!PrfFile || NewProfile) {
    if (PrfFile) fclose(PrfFile);
    PrfSetVersion(VE);
    vMsg("creating ambient profiles (IBJprf %s, profile version %d).", 
      PrfGetPrgVersion(), PrfGetVersion());
    SetAmbPrf();
    }
  else {
    if (PrfFile) fclose(PrfFile);
    ReadAmbPrf();                                                    eG(1);
    vMsg("profile file %s read in (%d lines).", PrfName, NZ);
    }
  DS = SD*SC;
  TE = SE*SC;
  DP = TE/SN;
  XX = XQ;
  YY = YQ;
  ZZ = HQ;
  SetAmb(XX, YY, ZZ);
  HU = UL;
  SS = 0.;
  LL = 0.;
  WW = 0.;
  ZT = 0.;
  RR = DQ/2.;
  AA = 0.;
  UU = UQ;
  JJ = IQ*UU;
  W1 = A1*PIFAC/180.;
  W2 = A2*PIFAC/180.;
  KK(1) = fabs(cos(W1)*UU)*cos(W2);
  KK(2) = fabs(cos(W1)*UU)*sin(W2);
  KK(3) = sin(W1)*UU;
  for (i=1; i<=3; i++) if (fabs(KK(i)) < 1.e-10) KK(i) = 0.;
  if (FQ > 0.) {
    TT = TL/(1. - UQ*UQ/(FQ*GRAV*DQ/2.));
    TQ = TT-TZERO;
    if ((TT < 0.) || (TT > 3000.+TZERO))                             eX(30);
  }
  else
    TT = TQ+TZERO;
  if (RQ > 0.) {
    QI = RQ * GetQs(TT, PL);
    SQ = QI;
  }
  else {
    QI = SQ;
    RQ = QI/GetQs(TT, PL);
  }
  if (RQ > 1.) {
    LQ += QI - GetQs(TT, PL);
    QI = GetQs(TT, PL);
    QQ = QI;
  }
  LI = LQ;
  ZI = QI + LI;
  if (LI > 0.) AA = RR;
  if (ZI > 1.)                                                      eX(31);
  DD = GetDensity(TT, QI, LI, PL);
  FQ = UQ*UQ*DL/((DL-DD)*DQ/2.*GRAV);
  CC = CQ;
  return 0;
eX_1:  eMSG("Error occured (%s)\n", ePGMs);
eX_10: eMSG("Parameter hq must be > 0 (%s)\n", ePGMs);
eX_11: eMSG("Parameter uq must be >= 0 (%s)\n", ePGMs);
eX_12: eMSG("Parameter iq must be >= 0 (%s)\n", ePGMs);
eX_13: eMSG("Parameter tq must be > %6.2f (%s)\n",-TZERO, ePGMs);
eX_14: eMSG("Parameter dq must be > 0 (%s)\n", ePGMs);
eX_15: eMSG("Parameter qq must be between 0 and 1 (%s)\n", ePGMs);
eX_16: eMSG("Parameter rq must be between 0 and 1 (%s)\n", ePGMs);
eX_17: eMSG("Parameter sq must be between 0 and 1 (%s)\n", ePGMs);
eX_19: eMSG("Parameter cq must be >= 0 (%s)\n", ePGMs);
eX_20: eMSG("Parameter ua must be >= 0 (%s)\n", ePGMs);
eX_21: eMSG("Parameter ta must be > %6.2f (%s)\n",-TZERO, ePGMs);
eX_22: eMSG("Parameter ra must be between 0 and 1 (%s)\n", ePGMs);
eX_23: eMSG("Parameter ia must be >= 0 (%s)\n", ePGMs);
eX_24: eMSG("Parameter ca must be >= 0 (%s)\n", ePGMs);
eX_25: eMSG("Parameter sc must be > 0 (%s)\n", ePGMs);
eX_26: eMSG("Parameter sd must be > 0 (%s)\n", ePGMs);
eX_27: eMSG("Parameter sn must be > 0 (%s)\n", ePGMs);
eX_28: eMSG("Parameter sp must be > 0 (%s)\n", ePGMs);
eX_29: eMSG("Pressure pa must be > 0 (%s)\n", ePGMs);
eX_30: eMSG("Froude number leads to invalid temperature (%s)\n", ePGMs);
eX_31: eMSG("Total specific water content larger than 1,0 (%s)\n", ePGMs);
eX_32: eMSG("Parameter uh must be > 0 (%s)\n", ePGMs);
//eX_33: eMSG("Parameter hm must be > 0 (%s)\n", ePGMs);
eX_34: eMSG("Parameter z0 must be > 0 (%s)\n", ePGMs);
}

//---------------------------------------------------------------------- Derivs
// sets the derivatives
int Derivs( double snew, double *var, double *dvds)
{
   double f, m, tmp, sig, rv, u2, ul2, r2, dt;
   m       = sqrt(var[2]*var[2] + var[3]*var[3] + var[4]*var[4]);
   UU      = m/var[1];
   dt      = (snew-SS)/UU;
   u2      = UU*UU;
   KK(1)   = var[2]/var[1];
   KK(2)   = var[3]/var[1];
   KK(3)   = var[4]/var[1];
   XX      = var[9]  + KK(1)*dt;
   YY      = var[10] + KK(2)*dt;
   ZZ      = var[11] + KK(3)*dt;
   LL      = var[12];
   ZI      = var[6]/var[1]; 
   JJ      = sqrt(fabs(var[8]/var[1]));
   ZT      = var[13];
   SetAmb(XX, YY, ZZ);
   SetTmpSigA(var, &tmp, &sig, &rv);
   TT      = tmp;
   LI      = sig;
   AA      = rv;
   QI      = ZI - LI;
   DD      = GetDensity(TT, QI, LI, PL);
   RR      = sqrt( fabs(var[1]/(DD*UU)) );
   r2      = RR*RR;
   CC      = var[7]/(r2*UU);
   if (SigC < SIGC) {
     f = r2/(DQ*DQ*0.25);
     SigC = (f < SIGC) ? f : SIGC;
   }
   dvds[1] = 2. * DL * GetEnt();
   dvds[2] = dvds[1]*KL(1);
   dvds[3] = dvds[1]*KL(2);
   dvds[4] = dvds[1]*KL(3) + r2*(DL-DD)*GRAV;
   f       = (KK(1)*KL(1) + KK(2)*KL(2) + KK(3)*KL(3))/u2;
   ul2     = UL*UL;
   dvds[5] = -(GRAV/CPRESS)*var[1]*KK(3)/UU*DL/DD + dvds[1]*(TL - LHEAT/CPRESS*SL);
   dvds[6] = dvds[1]*ZL;
   dvds[7] = dvds[1]*CL/DL;
   dvds[8] = dvds[1]*(u2 + ul2 - 2*f*u2 + JL*JL);
   dvds[9]  = KK(1)/UU;
   dvds[10] = KK(2)/UU;
   dvds[11] = KK(3)/UU - VS/UU;
   dvds[12] = sqrt(KK(1)*KK(1) + KK(2)*KK(2))/UU;
   dvds[13] = 1./UU;
   return 0;
}

//------------------------------------------------------------------ RungeKutta
// performes the Runge-Kutta integration
int RungeKutta(double *y, double *dydx, int n, double x, double h, double *yout)
{  int i;
   double xh, h1, h2, dym[20], dyt[20], yt[20];
   h1  = h*0.5;
   h2  = h/6.;
   xh  = x+h1;
   for (i=1; i<=n; i++) yt[i] = y[i] + h1*dydx[i];
   Derivs(xh, yt, dyt);
   for (i=1; i<=n; i++) yt[i] = y[i] + h1*dyt[i];
   Derivs(xh, yt, dym);
   for (i=1; i<=n; i++) {
     yt[i] = y[i] + h*dym[i];
     dym[i] += dyt[i];
   }
   Derivs(x+h, yt, dyt);
   for (i=1; i<=n; i++)
     yout[i] = y[i] + h2*(dydx[i] + dyt[i] + 2.*dym[i]);
  return 0;
}

//------------------------------------------------------------ GetTmpImplicitly
// calculates the plume temperature implicitly
double GetTmpImplicitly(double hh, double zz, double q0, double t0)
{ double aa, bb, cc, f, a, f2, arg;
  // note: hh = enthalphy/cp;
  //       zz = specific total water content;
  // what happens:
  //    Equation h = cp*T - hv*(zeta - qs(T));
  //    Clausius-C. qs = q0 exp(hv*(T-T0)/(R*T0*T0));
  //    Expansion of qs to second order -> quadratic equation for T;
  f = LHEAT/RVAPOUR/t0/t0;
  a = LOCP*q0;
  f2 = f*f;
  aa = a*0.5*f2;
  bb = 1 + a*f - a*f2*t0;
  cc = a - a*f*t0 + a*0.5*f2*t0*t0 - hh - LOCP*zz;
  arg = bb*bb - 4*aa*cc;
  if (arg < 0.) return -9999.;
  return (-bb + sqrt(arg))/(2*aa);
}

//------------------------------------------------------------------- SetTmpSigA
// calculates the plume temperature, liquid water content, and visible radius
int SetTmpSigA(double *var, double *tmp, double *sig, double *rv)
{ double zst, hst, zi, hi, b;
  double ee, eef, eer;
  double dr, r, f, t, tt, q0, eta, a, t0;
  // if the plume is dry, piece of cake
  if (ZI == 0.) {
    *sig = 0.;
    *rv  = 0.;
    *tmp = var[5]/var[1];
    return 0;
  }
  // model in a simple way zone of flow establishment, so that initially
  // the calculated plume properties do not differ from the given settings.
  ee = exp(-SS/2./DQ);
  eef = 1. + ee*(1-ee)*(sqrt(PIFAC)-2.);
  eer = RR/sqrt(SigC)*ee;
  b = RR/sqrt(SigC)*(1-ee);
  hst = SigC*(var[5]/var[1] - TL)/eef;
  zst = SigC*(ZI - ZL)/eef;
  // qs(T) is expanded around t0/q0 in GetTmpImplicitly()
  t0 = SigC*(TT - TL)/eef + TL;
  q0 = GetQs(t0, PL);
  // start from outside the plume, decrease the distance to the plume
  // axis, find the visible plume radius and integrate the liquid water
  // content
  eta = 0.;
  a = 0.;
  dr = b/40.;
  tt = t0;
  t = t0;
  hi = TL + hst;
  for (r=2*b; r>-dr; r-=dr) {
    f = (b > 0.) ? exp(-r*r/b/b) : 1.;
    hi = TL + hst*f;
    zi = ZL + zst*f;
    tt = GetTmpImplicitly(hi, zi, q0, t0);
    if ((tt-hi) > 0. && a == 0.) a = eer + r;
    if (a > 0.) {
      if (tt > hi) t = tt;
      if (t > hi)
        eta += (t-hi)*(eer + r);
    }
  }
  eta *= 2*dr; 
  if (t > hi && a > 0.) // -2003-05-14 only for a > 0
    eta += (t - hi)*eer*eer;
  eta  /= LOCP*RR*RR;
  *sig = eta;
  *rv = a;
  if (*sig < 1.e-10) { *sig = 0.; *rv = 0.; }
  *tmp = var[5]/var[1] + LOCP*(*sig);
  return 0;
}

//------------------------------------------------------------------------ CatF
// routine for the parameter output
int CatF(char *dsc, char *var)
{
  char s[80];
  sprintf(s, "%s%s", dsc, &(PrmFind(var,"?")->output_format[1]));
  strcat(PlrOutFormat, s);
  return 0;
}

//----------------------------------------------------------------- WriteEntry1
// routine for the parameter output
int WriteEntry1(TXTSTR *cp, char *var)
{
  char s[80];
  PRMTABREC *rec;
  dP(WriteEntry1);
  *s=0;
  rec = PrmFind(var, "?");
  if (rec == NULL)                                                   eX(1);
  PrmPrnRec(s, 80, rec);
  TxtCat(cp, s);
  return 0;
eX_1: eMSG("Can't find parameter %5s (%s)\n", var, ePGMs);
}

//----------------------------------------------------------------- WriteEntry2
// routine for the parameter output
int WriteEntry2(TXTSTR *cp, char *var)
{
  char *buf, s[80];
  int i;
  buf = TxtAlloc(5000);
  strcpy(buf, var);
  for (i=1; i<=NZ; i++) {
     if (!strcmp(var, "zv"))
       sprintf(s, PrmFind(var,"?")->output_format, ZV(i));
     if (!strcmp(var, "dv"))
       sprintf(s, PrmFind(var,"?")->output_format, DV(i));
     if (!strcmp(var, "sv"))
       sprintf(s, PrmFind(var,"?")->output_format, SV(i));
     if (!strcmp(var, "uv"))
       sprintf(s, PrmFind(var,"?")->output_format, UV(i));
     if (!strcmp(var, "xv"))
       sprintf(s, PrmFind(var,"?")->output_format, XV(i));
     if (!strcmp(var, "yv"))
       sprintf(s, PrmFind(var,"?")->output_format, YV(i));
     if (!strcmp(var, "wv"))
       sprintf(s, PrmFind(var,"?")->output_format, WV(i));
     if (!strcmp(var, "jv"))
       sprintf(s, PrmFind(var,"?")->output_format, JV(i));
     if (!strcmp(var, "tv"))
       sprintf(s, PrmFind(var,"?")->output_format, TV(i)-TZERO);
     if (!strcmp(var, "pv"))
       sprintf(s, PrmFind(var,"?")->output_format, PV(i));
     if (!strcmp(var, "qv"))
       sprintf(s, PrmFind(var,"?")->output_format, QV(i));
     if (!strcmp(var, "cv"))
       sprintf(s, PrmFind(var,"?")->output_format, CV(i));
     strcat(buf, "\t"); strcat(buf, s);
     }
  strcat(buf, "\n"); TxtCat(cp, buf);
  return 0;
}

//------------------------------------------------------------------- WriteData
// write out all parameters
int WriteData(int argc, char *argv[]) { 
  char s[128], *buf;
  int i;
  time_t ti;
  TXTSTR usrhdr = { NULL, 0 };
  dP(WriteData);
  TxtClr(&usrhdr);
  buf = TxtAlloc(20000);
  sprintf(s, "- IBJpluris, Version %s, compiled at %s %s\n", 
    VERSION, __DATE__, __TIME__);
  TxtCat(&usrhdr, s); 
  time(&ti);
  sprintf(s, "- %s", ctime(&ti));
  TxtCat(&usrhdr, s);
  for (i=1; i<argc; i++) {
    sprintf(s, "- %s\n", argv[i]);
    TxtCat(&usrhdr, s);
  }
  TxtCat(&usrhdr, "- "); TxtCat(&usrhdr, sFinal); TxtCat(&usrhdr, "\n");  
  TxtCat(&usrhdr, "-\n");  
  TxtCat(&usrhdr, "- Source\n");
  WriteEntry1(&usrhdr, "id"); WriteEntry1(&usrhdr, "hq");
  WriteEntry1(&usrhdr, "dq"); WriteEntry1(&usrhdr, "a1");
  WriteEntry1(&usrhdr, "a2"); WriteEntry1(&usrhdr, "fq");
  WriteEntry1(&usrhdr, "tq"); WriteEntry1(&usrhdr, "qq"); 
  WriteEntry1(&usrhdr, "uq");
  WriteEntry1(&usrhdr, "iq"); WriteEntry1(&usrhdr, "rq");
  WriteEntry1(&usrhdr, "lq"); WriteEntry1(&usrhdr, "sq");
  WriteEntry1(&usrhdr, "cq"); WriteEntry1(&usrhdr, "vs");
  TxtCat(&usrhdr, "- Environment\n");
  WriteEntry1(&usrhdr, "ha"); WriteEntry1(&usrhdr, "ua");
  WriteEntry1(&usrhdr, "hu"); WriteEntry1(&usrhdr, "us");
  WriteEntry1(&usrhdr, "uh"); WriteEntry1(&usrhdr, "ux");  
  WriteEntry1(&usrhdr, "da");
  WriteEntry1(&usrhdr, "th"); WriteEntry1(&usrhdr, "ta");
  WriteEntry1(&usrhdr, "tx");
  WriteEntry1(&usrhdr, "lm");  WriteEntry1(&usrhdr, "km");
  WriteEntry1(&usrhdr, "hm");
  WriteEntry1(&usrhdr, "z0"); WriteEntry1(&usrhdr, "vs");
  WriteEntry1(&usrhdr, "za"); WriteEntry1(&usrhdr, "nz");
  WriteEntry1(&usrhdr, "ia"); WriteEntry1(&usrhdr, "ra");
  WriteEntry1(&usrhdr, "ca"); WriteEntry1(&usrhdr, "ef");
  WriteEntry1(&usrhdr, "hu"); WriteEntry1(&usrhdr, "ve");
  TxtCat(&usrhdr, "- Calculation\n");
  WriteEntry1(&usrhdr, "sc"); WriteEntry1(&usrhdr, "sd");
  WriteEntry1(&usrhdr, "sn"); WriteEntry1(&usrhdr, "oo");
  TxtCat(&usrhdr, "- Profiles\n");
  WriteEntry2(&usrhdr, "zv");
  WriteEntry2(&usrhdr, "uv"); WriteEntry2(&usrhdr, "dv");
  WriteEntry2(&usrhdr, "sv");
  WriteEntry2(&usrhdr, "jv"); WriteEntry2(&usrhdr, "tv");
  WriteEntry2(&usrhdr, "qv"); WriteEntry2(&usrhdr, "cv");
  WriteEntry2(&usrhdr, "xv"); WriteEntry2(&usrhdr, "yv");
  WriteEntry2(&usrhdr, "wv"); WriteEntry2(&usrhdr, "pv");
  TxtCat(&usrhdr, "-\n");
  sprintf(buf, "mode\t%s\nartp P\n", "text"); 
  TxtCat(&usrhdr, buf);
  sprintf(buf, "form\t%s\n", PlrOutFormat);
  TxtCat(&usrhdr, buf);
  DmnWrite(OutName, usrhdr.s, &PlrOut);                               eG(2);
  return 0;
eX_2:
  eMSG("Can't write data (%s)\n", ePGMs);
}

//----------------------------------------------------------------- SetData
// store the actual parameters into the parameter structure
int SetData(int i)
{
  int j;
  double w1, w2, uh, sc;
  double tt = TT-TZERO;
  double rr, rc, x1, x2, z1, z2, co, si, h1, h2;
  double xmax=0, hmax, ts, lm, lb, hm, hb, fr, fq, kk, bf;
  dP(SetData);
  if (i == 0) {
    switch(OO) {
      case 0:
        AryCreate(&PlrOut, sizeof(PLROUT0), 1, 0, SN);               eG(1);
        CatF("ll","ll"); 
        CatF("hh","zz"); 
        CatF("hf","zz"); 
        CatF("vdi","zz");     
        CatF("lst","zz");  
        hmax = -1;
        break;      
      case 1:
        AryCreate(&PlrOut, sizeof(PLROUT1), 1, 0, SN);               eG(1);
        CatF("xx","xx"); CatF("yy","yy"); CatF("zz","zz");
        CatF("hh","hh"); CatF("ss","ss"); CatF("rr","rr");
        CatF("aa","aa"); CatF("uu","uu"); CatF("jj","jj"); 
        CatF("tt","tt"); CatF("cc","cc"); CatF("zt","zt");
        break;
      case 2:
        AryCreate(&PlrOut, sizeof(PLROUT2), 1, 0, SN);               eG(1);
        CatF("xx","xx"); CatF("yy","yy"); CatF("zz","zz");
        CatF("hh","hh"); CatF("ss","ss");
        CatF("ll","ll"); CatF("ww","ww");
        CatF("rr","rr"); CatF("uu","uu"); CatF("jj","jj");
        CatF("tt","tt"); CatF("cc","cc"); CatF("dd","dd");
        CatF("ee","ee"); CatF("w1","w1"); CatF("w2","w2");
        CatF("aa","aa"); 
        CatF("qi","qi"); 
        CatF("li","li"); 
        CatF("zi","zi");
        CatF("rc/sc","ff"); CatF("c/c0","ff"); CatF("c0/c","ff");
        CatF("ll/sc","ll");
        CatF("ww/sc","ww"); CatF("zz/sc","zz"); CatF("hh/sc","hh");
        CatF("w1r/sc","ff");CatF("z1r/sc","ff");CatF("h1r/sc","ff");
        CatF("w2r/sc","ff");CatF("z2r/sc","ff");CatF("h2r/sc","ff");
        CatF("w1a/sc","ff");CatF("z1a/sc","ff");CatF("h1a/sc","ff");
        CatF("w2a/sc","ff");CatF("z2a/sc","ff");CatF("h2a/sc","ff");
        break;
      case 3:
        AryCreate(&PlrOut, sizeof(PLROUT3), 1, 0, SN);               eG(1);
        CatF("xx","xx"); CatF("yy","yy"); CatF("zz","zz");
        CatF("hh","hh"); CatF("ss","ss");
        CatF("ll","ll"); CatF("ww","ww");
        CatF("rr","rr");
        CatF("aa","aa");
        CatF("w1r","ff");CatF("z1r","ff");CatF("h1r","ff");
        CatF("w2r","ff");CatF("z2r","ff");CatF("h2r","ff");
        CatF("w1a","ff");CatF("z1a","ff");CatF("h1a","ff");
        CatF("w2a","ff");CatF("z2a","ff");CatF("h2a","ff");
        break;
      case 4:
        AryCreate(&PlrOut, sizeof(PLROUT3), 1, 0, SN);               eG(1);
        CatF("xx/sc","xx"); CatF("yy/sc","yy"); CatF("zz/sc","zz");
        CatF("hh/sc","hh"); CatF("ss/sc","ss");
        CatF("ll/sc","ll"); CatF("ww/sc","ww");
        CatF("rr/sc","rr");
        CatF("aa/sc","aa");
        CatF("w1r/sc","ff");CatF("z1r/sc","ff");CatF("h1r/sc","ff");
        CatF("w2r/sc","ff");CatF("z2r/sc","ff");CatF("h2r/sc","ff");
        CatF("w1a/sc","ff");CatF("z1a/sc","ff");CatF("h1a/sc","ff");
        CatF("w2a/sc","ff");CatF("z2a/sc","ff");CatF("h2a/sc","ff");
        break;
      case 5:
        AryCreate(&PlrOut, sizeof(PLROUT5), 1, 0, SN);               eG(1);
        CatF("ll","ll"); CatF("hh","hh");
        CatF("bm","hh"); CatF("bb","hh");     
        break;
      default:
        AryCreate(&PlrOut, sizeof(PLROUT1), 1, 0, SN);               eG(1);
    }
  }
  rc = RR*sqrt(log(2.))/sqrt(SigC);
  uh = sqrt(KK(1)*KK(1) + KK(2)*KK(2));
  if (uh > MINVAL) {
    w2 = acos(KK(1)/uh)/PIFAC*180;
    if (w2 < 0.) w2 += 360.;
  }
  else w2 = -999.;
  if (UU != 0.) w1 = asin(KK(3)/UU)*180./PIFAC;
  else          w1 = -999.;
  co = uh/UU;
  si = KK(3)/UU;
  WW = sqrt((XX-XQ)*(XX-XQ) + (YY-YQ)*(YY-YQ));
  HH = ZZ - HQ;
  if (OO == 0) {
    if (hmax < 0) {
      hmax = getRise_VDI_3782_3(1.e10, KM, HQ, DQ, UQ, HU, QQ, &xmax); 
      ts = 0.4*xmax/HU;
    }
    Out0 = &(OUT0(i));
    Out0->ll = LL;
    Out0->hh = HH;  
    
    Out0->hf = (FinalRise != 0) ? FinalRise : HH;
    Out0->vdi = getRise_VDI_3782_3(LL, KM, HQ, DQ, UQ, HU, QQ, &xmax); 
    Out0->lst = (ts > 0) ? hmax*(1 - exp(-ZT/ts)) : hmax;
  }
  else if (OO == 1 || OO > 5 || OO < 1) {
    Out1 = &(OUT1(i));
    Out1->xx=XX; Out1->yy=YY; Out1->zz=ZZ;
    Out1->ss=SS; Out1->hh=HH; Out1->rr=RR;
    Out1->uu=UU; Out1->jj=JJ; Out1->tt=tt;
    Out1->aa=AA; Out1->cc=CC; Out1->tm=ZT;
  }
  else if (OO == 2) {
    Out2 = &(OUT2(i));
    Out2->xx=XX; Out2->yy=YY; Out2->zz=ZZ;
    if ((strncmp(ID,"ga20",4)==0)||(strncmp(ID,"ga18",4)==0))
      Out2->hh=1.26*HH;
    else
      Out2->hh=HH;
    Out2->ss=SS; Out2->ll=LL; Out2->ww=WW; Out2->rr=RR;
    Out2->uu=UU;
    Out2->jj=JJ; Out2->tt=tt; Out2->cc=CC;
    Out2->dd=DD;
    Out2->ee=GetEnt();
    Out2->w1=w1; Out2->w2=w2;
    Out2->aa=AA;
    Out2->qi=QI;
    Out2->li=LI;
    Out2->zi=ZI;
    Out2->rc=rc/SC;
    Out2->c0= (CQ != 0) ? CC/CQ : 0;
    Out2->c1= (CC != 0) ? CQ/CC : 0;
    for (j=1; j<=2; j++) {
      rr = (j==1) ? RR : AA;
      if (rr == RR) rr /= sqrt(SigC);
      x1 = WW+rr*si;
      z1 = ZZ-rr*co;
      h1 = z1 - HQ;
      x2 = WW-rr*si;
      z2 = ZZ+rr*co;
      h2 = z2 - HQ;
      sc = SC;
      if (j==1) {
        Out2->ls=LL/sc;
        Out2->ws=WW/sc; Out2->zs=ZZ/sc; Out2->hs=Out2->hh/sc;
        Out2->w1r=x1/sc; Out2->z1r=z1/sc; Out2->h1r=h1/sc;
        Out2->w2r=x2/sc; Out2->z2r=z2/sc; Out2->h2r=h2/sc;
      }
      else {
        Out2->w1a=x1/sc; Out2->z1a=z1/sc; Out2->h1a=h1/sc;
        Out2->w2a=x2/sc; Out2->z2a=z2/sc; Out2->h2a=h2/sc;
      }
    }
  }
  else if (OO == 3 || OO == 4) {
    sc = (OO == 3) ? 1. : SC;
    Out3 = &(OUT3(i));
    Out3->xx=XX/sc;
    Out3->yy=YY/sc;
    Out3->zz=ZZ/sc;
    Out3->hh=HH/sc;
    Out3->ss=SS/sc;
    Out3->ll=LL/sc;
    Out3->ww=WW/sc;
    Out3->rr=RR/sc;
    Out3->aa=AA/sc;
    for (j=1; j<=2; j++) {
      rr = (j==1) ? RR : AA;
      if (rr == RR) rr /= sqrt(SigC);
      x1 = WW+rr*si;
      z1 = ZZ-rr*co;
      z2 = ZZ+rr*co;
      h1 = z1 - HQ;
      x2 = WW-rr*si;
      h2 = z2 - HQ;
      if (j==1) {
        Out3->w1r=x1/sc; Out3->z1r=z1/sc; Out3->h1r=h1/sc;
        Out3->w2r=x2/sc; Out3->z2r=z2/sc; Out3->h2r=h2/sc;
      }
      else {
        Out3->w1a=x1/sc; Out3->z1a=z1/sc; Out3->h1a=h1/sc;
        Out3->w2a=x2/sc; Out3->z2a=z2/sc; Out3->h2a=h2/sc;
      }
    }
  }
  else if (OO == 5) { // comparison with Briggs formulae
    fq = (FQ < 0) ? -FQ : FQ;
    if (fq < 1.e-20)
     fq = 1.e-20;
    kk = UQ/HU;
    lm = DQ*kk;
    lb = DQ*kk*kk*kk/fq;
    bf = pow(3./(4.*ALPHA2*ALPHA2), 0.33333333);    
    Out5 = &(OUT5(i));
    Out5->ll = LL;
    Out5->hh = HH;
    Out5->bm = lm * bf * pow(LL/lm, 0.33333333);
    Out5->bb = lb * bf * pow(LL/lb, 0.66666666);
  }
  return 0;
eX_1:
  eMSG("Can't allocate arrays (%s)\n", ePGMs);
}


int checkFinalRise( void ) {
  if (FinalRise != 0) return 0;
  float w = fabs(KK(3));
  float u = sqrt(KK(1)*KK(1) + KK(2)*KK(2));
  float sw = 1.3*US; //PrfGetSigW(HA, LM, HM, US, Z0, 600.);
  int rc = 0;
  //printf("### h=%f, u=%f, v=%f, w=%f\n", ZZ-HQ, KK(1), KK(2), KK(3));
 // printf("### h=%f, z=%f, x=%f, y=%f, w=%f, sw=%f\n", ZZ-HQ, ZZ, XX, YY, w, sw);
  //printf("### LL=%6.0f, HH=%4.1f, w/u=%1.5f\n", LL, HH, w/u);
 if (u != 0 && w/u < 1./200.)
    rc = 1;
  else if (w < sw) {
    //printf("### h=%f, z=%f, x=%f, y=%f, w=%f, sw=%f\n", ZZ-HQ, ZZ, XX, YY, w, sw);
    rc = 2;
  }
  else
    return 0;
  sprintf(sFinal, "final rise according to VDI 3945/3 (%s): %1.1fm above source exit at distance %1.1fm", 
    (rc == 1) ? "1/200" : "sig-w", ZZ-HQ, sqrt((XX-XQ)*(XX-XQ) + (YY-YQ)*(YY-YQ)));  
  FinalRise = ZZ-HQ;
  return 0;
}


//------------------------------------------------------------------------- Run
int Run( void ) {
  dP(Run);
  int j;
  double sp;
  Initialize();                                                      eG(4);
  var[1] = RR*RR*DD*UU;
  var[2] = var[1] * KK(1);
  var[3] = var[1] * KK(2);
  var[4] = var[1] * KK(3);
  var[5] = var[1] * (TT - LHEAT/CPRESS*LI);
  var[6] = var[1] * ZI;
  var[7] = var[1] * CC/DD;
  var[8] = var[1] * JJ*JJ;
  var[9]  = XX;
  var[10] = YY;
  var[11] = ZZ;
  var[12] = LL;
  var[13] = 0.;
  SetData(0);                                                        eG(6); 
  j = 0;
  sp = 0.;
  while (j < SN) {
    RungeKutta(var, dvds, NEQ, SS, DS, var);
    if (ZZ-RR < 0. && CutOff) {
      vMsg("\nplume at ground, calculation stopped.");
      break;
    }
    if (Zmax > 0 && ZZ > Zmax) {
      vMsg("\nplume axis exceeding maximum height (%1.1fm), calculation stopped.", Zmax);
      break;
    }
    checkFinalRise();
    if (FinalRise != 0 && FinalRiseBreak == 1) {
      vMsg("\nfinal rise criteria reached, calculation stopped.");
      break;
    }
    sp += DS;
   // ZT += DS/(UL+UU);
    if (sp >= DP) {
      j++; sp=0.;
      SetData(j);                                                   eG(6);
      if (!MsgQuiet && MsgVerbose) {
        printf("\r             ");
        printf("\rstep %d", j); fflush(stdout);
      }
    }
    SS += DS;
    Derivs(SS, var, dvds);
  }
  return 0;
eX_4:
  eMSG("Error occured (%s)\n", ePGMs);
eX_6:
  eMSG("Error occured (SetData) (%s)\n", ePGMs);
}

//---------------------------------------------------------------- setParameter
int setParameter(char *line) {
  char *p1, *p2, s[64];
  void *p;
  PRMTABREC *pt;
  sprintf(s, "%s", line);
  
  p1 = strchr(s, '#');
  p2 = strchr(s, '=');
  if (p1) {
    if ((p2) && p2<p1)  p1 = p2;
  }
  else p1 = p2;
  if (!p1) return -1;
  *p1++ = 0;
  pt = PrmFind(s, "?"); 
  if (pt == NULL) return -2;
  if (pt->type & PRM_ISARRAY) return -3;    
  p2 = (char*)&(pt->u);
  if ((pt->type & PRM_ISANY) == PRM_ISSTRING)
    p = pt->u.p;
  else 
    p = p2;
  sscanf(p1, pt->input_format, p);
  return 0;
}


//---------------------------------------------------------- getRise_VDI-3782-3
double getRise_VDI_3782_3(double x, int km, double hq, double dq, double vq, 
double uq, double qq, double *xmax) {
  double h=-1, h1, h2, xmax1;
  if (qq > 1.4) {
    if (km == 5 || km == 6) {
      *xmax = (qq > 6) ? 288 * pow(qq, 0.4) : 195 * pow(qq, 0.625);
      if (x > *xmax) x = *xmax;
      h = 3.34 * pow(qq, 0.3333) * pow(x, 0.6667) / uq;
      if (h > 1100-hq) h = 1100-hq;
    }
    else if (km == 3 || km == 4) {
      *xmax = (qq > 6) ? 210 * pow(qq, 0.4) : 142 * pow(qq, 0.625);
      if (x > *xmax) x = *xmax;
      h = 2.84 * pow(qq, 0.3333) * pow(x, 0.6667) / uq;      
      //h1 = pow(3./(2*ALPHA2*ALPHA2), 0.3333) * pow(0.25*dq*dq*vq*GRAV*(TQ-TA)/(TQ+TZERO), 0.3333) * pow(x, 0.6667) / uq;
      if (h > 800-hq) h = 800-hq;
    }
    else if (km == 1 || km == 2) {    
      *xmax = (km == 1) ? 104 * uq : 127 * uq;
      if (x > *xmax) x = *xmax;
      h = 3.34 * pow(qq, 0.3333) * pow(x, 0.6667) / uq;
      h1 = getRise_VDI_3782_3(x, 3, hq, dq, vq, uq, qq, &xmax1); 
      if (h1 < h) {
        h = h1;
        *xmax = xmax1;
      }
    }
    else
      return -999;  
    
  }
  else {
    h1 = (0.35*uq*dq*vq + 84*sqrt(qq)) / uq;
    h2 = 3.0*vq*dq / uq;
    h = (h2 > h1) ? h2 : h1;  
    *xmax = 0;
  }
  return h;
}

//------------------------------------------------------------------------ Help
void Help( void ) {
  MsgVerbose = 1;
  vMsg("usage: IBJpluris <directory> [<option> ...]");
  vMsg("Version %s", VERSION);
  vMsg("Compiled at %s  %s", __DATE__, __TIME__);
  vMsg("options:");
  vMsg(
  "  -e       echo parameters to log file\n"
  "  -f       break at final rise according to VDI 3945/3 (using sw=1.3ustar)\n"                     
  "  -h       listing of all input parameters\n"
  "  -i<name> alternative command file name (lprcmd.txt)\n"
  "  -j<name> alternative profile file (extension: .dmna, profiles.dmna)\n"
  "  -l<name> alternative log file name (lprlog.txt)\n"
  "  -n       don't break if plume touches the ground\n"
  "  -p       create profiles (don't use profile file)\n"
  "  -q       quiet mode (equal to -v0)\n"  
  "  -v<verb> verbose level (1)\n"
  "  -z<zmax> break if plume axis exceeds height zmax (default: 1100m)\n"
  "  -D<param>[#=]<value> re-define a parameter value\n");
  exit(0);
}

//------------------------------------------------------------------------ Help
void HelpParam( void ) {
  MsgVerbose = 1;
  vMsg("%s",
   "Parameter drfen nur in den zugehrigen Abschnitten gesetzt werden.\n"
   "In der folgenden Tabelle sind alle Parameter mit Namen (zwei Buchstaben,\n"
   "Gro-/Kleinschreibung ist nicht signifikant), Standardwert, Vielfachheit\n"
   "(bei Vektoren) und ihrer Bedeutung aufgelistet: \n");
  PrmPrint("*a*c*o*q*e");
  exit(0);
}

//======================================================================== MAIN
int main(int argc, char *argv[]) {
  int i, j;
  time_t ti;
  char *s, cmd[80], arg[80], fname[512];
  dP(PlrMain);
  MsgVerbose = 1;
  if (argc < 2)  Help();
  
  // analyse options
  *InpName = 0;
  *PrfName = 0;
  strcpy(LogName, "plrlog.txt");
  for (i=1; i<argc; i++) {
    if (strchr(SYMOPT, *argv[i])) {
      s = argv[i]+1;
      switch (s[0]) {
        case 'e': PrmEcho = 1;
                  sscanf(s+1, "%x", &PrmEcho);
                  break;
        case 'n': CutOff=0;
                  break;
        case 'z': Zmax = ZMAX;
                  sscanf(s+1, "%lf", &Zmax);
                  if (Zmax < 0)
                    Zmax = ZMAX;
                  break;          
        case 'h': HelpParam();
                  break;
        case 'p': NewProfile = 1;
                  break;
        case 'f': FinalRiseBreak = 1;
                  break;
        case 'l': strcpy(LogName, s+1);
                  break;
        case 'q': MsgQuiet = 1;
                  sscanf(s+1, "%x", &MsgQuiet);
                  break;
        case 'i': strcpy(InpName, s+1);
                  break;
        case 'j': strcpy(PrfName, s+1);
                  NewProfile = 0;
                  break;
        case 'v': MsgVerbose = 1;
                  sscanf(s+1, "%x", &MsgVerbose);
                  break;
        case 'D': break;
        default:  printf("Unknown option %s\n", s);
                  Help();
      }
    }
  }  
  
  // set names
  sprintf(fname, "%s", argv[1]);
  for (s=fname; (*s); s++)  if (*s == '\\')  *s = '/';
  strcpy(Dir, fname);
  if (Dir[strlen(Dir)-1] != '/') strcat(Dir, "/");
  if (*PrfName == 0) {
    sprintf(PrfName, "%s%s", Dir, "profiles.dmna");
  }
  else {
    strcpy(fname, PrfName);
    sprintf(PrfName, "%s%s", Dir, fname);
  }
  if (*InpName == 0) {
    sprintf(InpName, "%s%s", Dir, "plrcmd.txt");
    sprintf(OutName, "%s%s", Dir, "plrcmd");
  }
  else {
    strcpy(fname, InpName);
    sprintf(InpName, "%s%s", Dir, fname);
    j = strlen(InpName)-1; 
    while (j >= 0) if (InpName[j--] == '.') break;
    if (j < 0) j = strlen(InpName);
    strcpy(OutName, Dir);
    strncpy(OutName, InpName, j+1);
  }   
  if (*LogName != '-') {
    strcpy(fname, LogName);
    sprintf(LogName, "%s%s", Dir, fname);    
    MsgFile = fopen(LogName, "w");  if (!MsgFile)                    eX(1);
  }
  vMsg("\nIBJpluris, Version %s from %s %s", VERSION, __DATE__, __TIME__);
  vMsg("Copyright Janicke Consulting, Ueberlingen, Germany, 1999-2010");
  vMsg("IBJpluris comes with ABSOLUTELY NO WARRANTY");
  vMsg("IBJpluris is free software under the GNU PUBLIC LICENCE\n");
  time(&ti);
  //vMsg("STARTING at %s", ctime(&ti));

  // read in parameter file  
  strcpy(fname, InpName);
  *cmd = 0;
  *arg = 0;
  while (i=PrmRead(fname, cmd, arg)) {
    if (i<0)                                                         eX(4);
    if (cmd[1]=='i')                                                 eX(3);
    if (tolower(cmd[1]) == 'e') break;
    *fname = 0;
  }
  vMsg("parameter file %s read in.", InpName);
  
  // analyse other options
  for (i=1; i<argc; i++) {
    if (strchr(SYMOPT, *argv[i])) {
      s = argv[i]+1;
      switch (s[0]) {
        case 'D': if (setParameter(s+1) < 0)                         eX(5);
                  break;
        default: ;
      }
    }
  }  
  // run and write out results
  Run();
  WriteData(argc, argv);                                             eG(2);
  vMsg("\rresults written to %s.%s.", OutName, "dmna");
  vMsg(sFinal);
  time(&ti);
  //vMsg("FINISHED at %s\n", ctime(&ti));
  exit(0);  
eX_1:
  eMSG("Working directory does not exist / Can't open log file %s (%s)", LogName, ePGMs);
eX_2:
  eMSG("Can't open output file %s (%s)", OutName, ePGMs);
eX_3:
  eMSG("Parameter section %s can not be used in the parameter file (%s)",
  cmd, ePGMs);
eX_4:
  eMSG("Error occured (%s)", ePGMs);
eX_5:
  eMSG("Invalid parameter definition %s (%s)", s, ePGMs);
}
