//======================================================== IBJprf.c
//
// Boundary layer profile functions for IBJ-programs
// =================================================
//
// Copyright (C) Ing.-Bro Janicke, berlingen, Germany, 2000-2010
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// last change:
// 2000-08-11 uj  1.3.1 Profile Version 3 (Degrazia)
// 2002-02-01 uj  1.3.2 hm for VDI 3783-8
// 2002-03-17 uj  1.3.3 K, Sigma const. for z < d0 +6*z0
// 2002-03-23 uj  1.3.4 VDI3783_8/AUSTAL2000 differ in Hm
// 2002-03-24 uj  1.3.5 Wind direction
// 2006-12-30 uj  1.3.6 more accurate PRF_TMPGRADIENT and PRF_GRAV
// 2006-12-30 uj  1.3.7 GetStability
// 2010-11-10 uj  1.4.1 Temperature profile from KTA
//
//==================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "IBJprf.h"

#define LASAT      1
#define VDI3783_8  2
#define AUSTAL2000 3

char *PrfVersion = "1.4.1";

float Ha, Ua, Z0, Ra, Ust, Lm, Hm, Zz[101], Interval, D0=0., Kappa=PRF_KAPPA;
float Beta = 1.0;
int   Nz = 100;
char  Name[512];
int   Version = AUSTAL2000;
int   ImplVersions[] = { LASAT, VDI3783_8, AUSTAL2000 };
int   NVersions = 3;
int   StdVersion = AUSTAL2000;
int   DEBUG = 0;

double z0vec[9] =
     { 0.01, 0.02, 0.05, 0.10, 0.20, 0.50,  1.0,  1.5,  2.0 };
double lmTAL[6][9] = {
     {    7,    9,   13,   17,   24,   40,   65,   90,  118 },
     {   25,   31,   44,   60,   83,  139,  223,  310,  406 },
     {99999,99999,99999,99999,99999,99999,99999,99999,99999 },
     {  -25,  -32,  -45,  -60,  -81, -130, -196, -260, -326 },
     {  -10,  -13,  -19,  -25,  -34,  -55,  -83, -110, -137 },
     {   -4,   -5,   -7,  -10,  -14,  -22,  -34,  -45,  -56 } };
int nz0 = 9;

double DTDZ_KTA[9][5] = {
    { -1.13, -1.03, -0.91, -0.37, 0.78 },
    { -1.18, -1.05, -0.91, -0.22, 1.12 },
    { -1.39, -1.18, -0.97, -0.16, 1.25 },
    { -1.61, -1.33, -1.00, -0.10, 1.32 },
    { -1.82, -1.48, -1.04, -0.04, 1.39 },
    { -2.03, -1.62, -1.08,  0.02, 1.46 },
    { -2.24, -1.77, -1.16,  0.08, 1.53 },
    { -2.45, -1.92, -1.25,  0.14, 1.60 },
    { -2.66, -2.07, -1.40,  0.20, 1.67 } };


//----------------------------------------------------------------- PrfSetKappa
void PrfSetKappa(float kappa) { Kappa = kappa; }

//-------------------------------------------------------------------- PrfSetD0
void PrfSetD0(float d0) { D0 = d0; }

//-------------------------------------------------------------------- PrfSetHa
void PrfSetHa(float ha) { Ha = ha; }

//-------------------------------------------------------------------- PrfSetRa
void PrfSetRa(float ra) { Ra = ra; }

//------------------------------------------------------------------ PrfSetBeta
void PrfSetBeta(float beta) { Beta = beta; }

//------------------------------------------------------------ PrfGetPrgVersion
char *PrfGetPrgVersion() { return PrfVersion; }

//--------------------------------------------------------------- PrfSetVersion
void PrfSetVersion(int version) {
  int i;
  Version = version;
  for (i=0; i<NVersions; i++) if (ImplVersions[i] == Version) break;
  if (i >= NVersions) Version = StdVersion;
  if (DEBUG) printf("Version set to %d\n", Version);
}

//--------------------------------------------------------------- PrfGetVersion
int PrfGetVersion() {
  return Version;
}

//---------------------------------------------------------------- PrfGetTmpKTA
float PrfGetTmpKTA(float z, float h0, float t0, float u10, int kl) { //-2010-11-10
  float u10round, g;
  int iu, kta, ktai;
  u10round = (int)(u10*10 + 0.5)/10.0;
  iu = (int)u10round;
  if (iu > 8)
    iu = 8;
  kta = 7 - kl;
  ktai = kta - 1;
  if (kta == 6)
    g = DTDZ_KTA[iu][ktai-1] + 0.5*(DTDZ_KTA[iu][ktai-1] - DTDZ_KTA[iu][ktai-2]);
  else if (kta == 1)
    g = DTDZ_KTA[iu][ktai] + 0.5*(DTDZ_KTA[iu][ktai] - DTDZ_KTA[iu][ktai+1]);
  else
    g = 0.5*(DTDZ_KTA[iu][ktai-1] + DTDZ_KTA[iu][ktai]);
  return t0 + 0.01*g*(z - h0);
}

//------------------------------------------------------------------- PrfGetTmp
float PrfGetTmp(float z, float lm, float ust, float z0, float mean, float tmp0) {
  float zeta, zeta0, psi, y, y0, theta, tst;
  zeta = (z+z0)/lm;
  zeta0 = z0/lm;
  tst = ust*ust*mean/(Kappa*PRF_GRAV*lm);
  if (lm > 0) {
    psi = -PRF_PS*(zeta - zeta0);
  }
  else {
    y = sqrt(1. - PRF_PL*zeta);
    y0 = sqrt(1. - PRF_PL*zeta0);
    psi = 2*log((1.+y)/(1.+y0));
  }
  theta = tmp0 + PRF_PN*tst/Kappa*(log(zeta/zeta0) - psi);
  return (theta + PRF_TMPGRADIENT*z);
}

//--------------------------------------------------------------------- PrfGetU
float PrfGetU(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, zeta, zeta0, psi, x1, x2, x3, x, x0, u, hh;
  if (Version == VDI3783_8 || Version == AUSTAL2000) hh = D0+6*z0;
  else hh = D0+D0;
  z = (h < hh) ? hh-D0 : h-D0;
  if (Version == VDI3783_8 || Version == AUSTAL2000) z -= z0;
  if (z < 0) z = 0;
  zeta = (z+z0)/lm;
  zeta0 = z0/lm;
  if (lm > 0.) {
    x1 = -5*zeta0;
    x2 = 7*log(2.) - log(zeta0) - 5*zeta0 - 4;
    x3 = 8*log(20) - 11.165 - log(2.) - log(zeta0) - 5*zeta0;
    if (zeta < 0.5)
      psi = log(zeta/zeta0) + 5*zeta + x1;
    else if (zeta <= 10.)
      psi = 8*log(zeta) + 4.25/zeta - 0.5/zeta/zeta + x2;
    else
      psi = 0.7585*zeta + x3;
    u = ust/Kappa*psi;
  }
  else {
    x = pow(1 - 15*zeta, 0.25);
    x0 = pow(1 - 15*zeta0, 0.25);
    x1 = (1+x)*(1+x)/(1+x0)/(1+x0) * (1+x*x)/(1+x0*x0);
    psi = log(x1) - 2*(atan(x) - atan(x0));
    u = ust/Kappa*(log(zeta/zeta0) - psi);
  }
  if (h < hh) u *= h/hh;
  if (u < 0) u = 0.;
  return u;
}

//------------------------------------------------------------------ PrfGetSigU
float PrfGetSigU(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, av, sig, wst;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (Version == VDI3783_8 || Version == AUSTAL2000) {
    if (lm > 0) {
      sig = 2.4*ust*exp(-VDI_EX*z/hm);
    }
    else {
      wst = ust*pow(-hm/Kappa/lm, 1./3.);
      sig = pow(2.4*ust, 3.) + pow(0.59*wst, 3.);
      sig = pow(sig, 0.333333) * exp(-VDI_EX*z/hm);
    }
  }
  else {
    if (interval < 600.) interval = 600.;
    if (interval > 3600.) interval = 3600.;
    av = pow(interval/3600., 0.2);
    sig = PRF_FU*av*ust;
    if (lm < 0.)
      sig *= pow(1. - 0.064*hm/lm, 1./3.);
  }
  if (sig < 0.01*ust) sig = 0.01*ust;
  return sig;
}

//------------------------------------------------------------------ PrfGetSigV
float PrfGetSigV(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, av, sig, wst;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (Version == VDI3783_8 || Version == AUSTAL2000) {
    if (lm > 0.) {
      sig = 1.8*ust*exp(-VDI_EX*z/hm);
    }
    else {
      wst = ust*pow(-hm/Kappa/lm, 1./3.);
      sig = pow(1.8*ust, 3.) + pow(0.59*wst, 3.);
      sig = pow(sig, 0.333333) * exp(-VDI_EX*z/hm);
    }
  }
  else {
    if (interval < 600.) interval = 600.;
    if (interval > 3600.) interval = 3600.;
    av = pow(interval/3600., 0.2);
    sig = PRF_FV*av*ust;
    if (lm < 0.)
      sig *= pow(1. - 0.064*hm/lm, 1./3.);
  }
  if (sig < 0.01*ust) sig = 0.01*ust;
  return sig;
}

//------------------------------------------------------------------ PrfGetSigW
float PrfGetSigW(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, sig, mu, wst, a;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  a = z/hm;
  if (Version == VDI3783_8 || Version == AUSTAL2000) { 
    if (lm > 0.) {
      sig = 1.3*ust*exp(-VDI_EX*a);
    }
    else {
      wst = ust*pow(-hm/Kappa/lm, 0.333333);
      sig = pow(1.3*ust*exp(-VDI_EX*a), 3.);
      if (1-0.8*a > 0.)
        sig += pow(1.3*pow(a,0.333333)*(1-0.8*a)*wst, 3.);
      sig = pow(sig, 0.333333);
    }
  }
  else {
    sig = PRF_FW*ust;
    mu = exp(-2*PRF_ALPHA*a);
    if (lm < 0.)
      sig *= pow(mu*mu*mu + 2.10*(-z/lm)*exp(-3*a), 0.333333);
    else
      sig *= mu;
  }
  if (sig < 0.01*ust) sig = 0.01*ust;
  return sig;
}

//-------------------------------------------------------------------- PrfGetKW
float PrfGetE(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, f1, f2, a, rwsus3;
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (hm <= 0)  hm = 800;
  a = z/hm;
  f1 = 2.5*Kappa*a;
  if (a < 1)  f1 += (1-a)*(1-a);
  if (lm < 0) {
    rwsus3 = -hm/(Kappa*lm);
    f2 = 1.5 - 1.3*pow(a, 1./3.);
    if (f2 > 0)  f1 += rwsus3*a*Kappa*f2;
    if (f1 < 1.) f1 = 1.;  
  }
  else { 
    f1 = 1. + 4*z/lm;
  }
  return (f1*ust*ust*ust/(Kappa*z));
}

//-------------------------------------------------------------------- PrfGetKW
float PrfGetKW(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, k, sig, kmin, e, a, f, sf;
  sig = PrfGetSigW(h, lm, hm, ust, z0, interval);
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  a = z/hm;
  if (Version == VDI3783_8 || Version == AUSTAL2000) {  
    e = PrfGetE(h, lm, hm, ust, z0, interval);
    k = 2*sig*sig*sig*sig/VDI_C0/e;
  }
  else {
    k = Kappa*ust*z/PRF_PN;
    f = z/lm;
    if (lm > 0.)
      k *= 1./(1 + PRF_PS*f) * exp(-6*PRF_ALPHA*a);
    else {
      sf = sqrt(0.01*hm/(-lm));
      a = ((1 - a) > 0.) ? (1-a)*(1-a) : 0.;
      k *= pow(exp(-18*PRF_ALPHA*z/hm) + (3*PRF_PS*(-f) + 120*f*f)*a*a*a, 0.333333);
    }
  }
  kmin = Beta*sig*sig*z0/ust;
  if (k < kmin) k = kmin;
  return k;
}

//-------------------------------------------------------------------- PrfGetKU
float PrfGetKU(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, av, u, sig, k, kmin, e;
  sig = PrfGetSigU(h, lm, hm, ust, z0, interval);
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (Version == VDI3783_8 || Version == AUSTAL2000) {
    e = PrfGetE(h, lm, hm, ust, z0, interval);
    k = 2*sig*sig*sig*sig/VDI_C0/e;
  }
  else {
    if (interval < 600.) interval = 600.;
    if (interval > 3600.) interval = 3600.;
    av = pow(interval/3600., 0.2);
    u = PrfGetU(h, lm, hm, ust, z0, interval);
    k = PRF_FFU*av*u/(10*PRF_FU*ust)*hm*sig;
  }
  kmin = Beta*sig*sig*z0/ust;
  if (k < kmin) k = kmin;
  return k;
}

//-------------------------------------------------------------------- PrfGetKV
float PrfGetKV(float h, float lm, float hm, float ust, float z0, float interval) {
  float z, av, u, sig, k, kmin, e;
  sig = PrfGetSigV(h, lm, hm, ust, z0, interval);
  z = (h < D0+6*z0) ? 6*z0 : h-D0;
  if (Version == VDI3783_8 || Version == AUSTAL2000) {
    e = PrfGetE(h, lm, hm, ust, z0, interval);
    k = 2*sig*sig*sig*sig/VDI_C0/e;
  }
  else {
    if (interval < 600.) interval = 600.;
    if (interval > 3600.) interval = 3600.;
    av = pow(interval/3600., 0.2);
    u = PrfGetU(h, lm, hm, ust, z0, interval);
    k = PRF_FFV*av*u/(10*PRF_FV*ust)*hm*sig;
  }
  kmin = Beta*sig*sig*z0/ust;
  if (k < kmin) k = kmin;
  return k;
}

//------------------------------------------------------------- PrfGetStability
// sets Lm from a given Klug/Manier class and vice versa
int PrfGetStability( float z0, float *_lm, int *_km ) {
    double lmo, lz0, a1, a2, lz1, lz2;
    double rr[6], ll[6];
    int i, k, i1, i2;
    int n0 = nz0;
    if (z0 < z0vec[0])  z0 = z0vec[0];
    if (z0 > z0vec[n0-1])  z0 = z0vec[n0-1];
    for (i2=1; i2<n0; i2++) {
      if (z0 <= z0vec[i2])  break;
    }
    i1 = i2-1;
    lz0 = log(z0);
    lz1 = log(z0vec[i1]);
    lz2 = log(z0vec[i2]);
    a1 = (lz2 - lz0)/(lz2 - lz1);
    a2 = (lz0 - lz1)/(lz2 - lz1);
    for (k=0; k<6; k++) {
      rr[k] = a1/lmTAL[k][i1] + a2/lmTAL[k][i2];
      ll[k] = 1/rr[k];
    }
    if (*_lm == 0) {
      if (_km == NULL || *_km <= 0) return -1;
      if (*_km <= 0 || *_km > 6)  
          if (*_km == 7)
            *_lm = ll[2];
          else 
            *_lm = 0;
        else
          *_lm = ll[*_km-1];
    }
    else {
      if (_lm == NULL || *_lm == 0) return -2;
      for (k=0; k<5; k++)  
        if (1/(*_lm) > 0.5*(rr[k] + rr[k+1]))  
          break;
      *_km = k+1;
    }
    return 0;
}


//-------------------------------------------------------------------- PrfGetHM
float PrfGetHM(int km, float lm, float ust) {
  float hm, a;
  if (Version == VDI3783_8) {
    hm = 1100.;
    if (lm > 0.) {
      hm = 0.7*sqrt(ust*lm/1.11e-4);
      if (hm > 800.) hm =  800.;
      if (hm < 250.)  hm = 250.;
    }
  }
  else if (Version == AUSTAL2000) {
    if (km >= 5)   
      hm = 1100;
    else
      hm = 800;
    if (lm > 0.) {
      hm = 0.3*ust/1.e-4;
      a = sqrt(1.e-4*lm/ust);
      if (a < 1)  hm *= a;
      if (hm > 800)  hm = 800;   
    }
  }
  else {    
    if (lm > 0.) {
      hm = PRF_ALPHA*ust/PRF_FCOR;
      if (lm < ust/PRF_FCOR) hm *= sqrt(PRF_FCOR*lm/ust);
    }
    else {
      if (lm < -200) hm = 800.;
      else           hm = 1100.;
    }
  }
  return hm;
}

//-------------------------------------------------------------------- PrfGetRa
float PrfGetRa(float h, float lm, float hm, float z0) {
  float zp, za, dmax, dh, hol, d;
  if (Version != AUSTAL2000 || lm == 0.) return Ra;
  dmax = 45.;
  hol = hm/lm;
  if (hol < -10.)    dh = 0;                   
  else if (hol < 0.) dh = dmax*(1 + 0.1*hol);   
  else               dh = dmax;  
  if (h > hm) h = hm;
  zp = (h  < D0+6*z0) ? 6*z0 : h-D0;
  za = (Ha < D0+6*z0) ? 6*z0 : Ha-D0;
  d = Ra + 1.23*dh*(exp(-1.75*za/hm) - exp(-1.75*zp/hm));
  return d;
}  

#ifdef MAIN /*###############################################################*/
#define  ISOPTION(a)   ((a) == '-')

//-------------------------------------------------------------------- profiles
void profiles() {
  int i;
  float vx[40], su[40], sv[40], sw[40], ku[40], kv[40], kw[40], ra[40], tm[40], z;
  float u10;
  int kl;
  FILE *fp;

  if (Nz <= 0) {
    Nz = 30;
    Zz[0] = 0.;
    i = 1;
    while (i <= 20) { Zz[i] = Zz[i-1] + 10.; i++; }
    while (i <= 30) { Zz[i] = Zz[i-1] + 50.; i++; }
    while (i <= 40) { Zz[i] = Zz[i-1] + 50.; i++; }
  }
  if (Ust == 0.) {
    if (Ha <= 0.) Ha = 10.;
    if (Ua <= 0.) Ua = 4.;
  }
  if (Ra <= 0.) Ra = 270.;
  if (Z0 <= 0.) Z0 = 1.0;
  if (Lm == 0.) Lm = 99999.;
  if (Ust <= 0.) Ust = Ua/PrfGetU(Ha, Lm, Hm, 1., Z0, Interval);
  
  u10 = PrfGetU(10.0, Lm, Hm, Ust, Z0, Interval);
  PrfGetStability(Z0, &Lm, &kl);

  if (Hm <= 0.) Hm = PrfGetHM(kl, Lm, Ust);

  for (i=0; i<=Nz; i++) {
    z = Zz[i];
    vx[i] = PrfGetU(z, Lm, Hm, Ust, Z0, Interval);
    su[i] = PrfGetSigU(z, Lm, Hm, Ust, Z0, Interval);
    sv[i] = PrfGetSigV(z, Lm, Hm, Ust, Z0, Interval);
    sw[i] = PrfGetSigW(z, Lm, Hm, Ust, Z0, Interval);
    ku[i] = PrfGetKU(z, Lm, Hm, Ust, Z0, Interval);
    kv[i] = PrfGetKV(z, Lm, Hm, Ust, Z0, Interval);
    kw[i] = PrfGetKW(z, Lm, Hm, Ust, Z0, Interval);
    ra[i] = PrfGetRa(z, Lm, Hm, Z0);
    tm[i] = PrfGetTmpKTA(z, 2.0, 10+273.15, u10, kl);              //-2010-11-10
    //tm[i] = PrfGetTmp(z, Lm, Ust, Z0, 10+273.15, 10+273.15);
  }

  remove(Name);
  fp = (strlen(Name) > 1) ? fopen(Name, "w") : stdout;
  fprintf(fp, "   ' IBJprf %s profile version %d\n", PrfVersion, Version);
  fprintf(fp, "   ' us=%8.4f lm=%8.2f hm=%8.2f\n", Ust, Lm, Hm);
  fprintf(fp, "   ' nz=%8.0d ha=%8.2f ua=%8.3f\n", Nz, Ha, Ua);
  fprintf(fp, "   ' z0=%8.4f d0=%8.2f in=%8.2f\n", Z0, D0, Interval);
  fprintf(fp, "   ' ra=%8.4f kp=%8.2f Beta=%8.3f\n", Ra, Kappa, Beta);
  fprintf(fp, "ha  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", Zz[i]); fprintf(fp, "\n");
  fprintf(fp, "ua  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", vx[i]); fprintf(fp, "\n");
  fprintf(fp, "su  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", su[i]); fprintf(fp, "\n");
  fprintf(fp, "sv  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", sv[i]); fprintf(fp, "\n");
  fprintf(fp, "sw  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", sw[i]); fprintf(fp, "\n");
  fprintf(fp, "ku  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", ku[i]); fprintf(fp, "\n");
  fprintf(fp, "kv  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", kv[i]); fprintf(fp, "\n");
  fprintf(fp, "kw  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", kw[i]); fprintf(fp, "\n");
  fprintf(fp, "ra  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", ra[i]); fprintf(fp, "\n");
  fprintf(fp, "ta  ");
  for (i=0; i<Nz; i++) fprintf(fp, " %11.3e", tm[i]-273.15); fprintf(fp, "\n");
}

//------------------------------------------------------------------------ Help
void Help() {
  printf("Calculation of 1-dim. meteorological standard profiles \n");
  printf("usage: IBJprf [options]\n");
  printf("options:\n");
  printf("  -d<number>   Displacement height (m, default: 0)\n");
  printf("  -i<number>   Interval (s, default: 3600)\n");
  printf("  -k<number>   Kappa (default: 0.4)\n");
  printf("  -l<number>   Monin-Obukhov length (m, default: 99999)\n");
  printf("  -m<number>   Height of the mixing layer (m)\n");
  printf("  -h<number>   Anemometer height (m, default: 10)\n");
  printf("  -u<number>   Wind velocity at anemometer height (m/s, default: 4)\n");
  printf("  -s<number>   Shear velocity (m/s)\n");
  printf("  -z<number>   Roughness length (m, default: 1)\n");
  printf("  -r<number>   Wind direction at anemometer height (deg, default: 270)\n");
  printf("  -v<z1>,<z2>,...  Vertical grid (m)\n");
  printf("  -f<file>     Output file name\n");
  printf("  -V<version>  Version (1:LASAT 2.9 2:VDI-3783/8 3:AUSTAL2000)\n");
  exit(0);
}

//------------------------------------------------------------------------ main
int  main( int argc, char *argv[] ) {
  int i, j;
  float z;
  char *pc;
  //printf("IBJprf, version %s of %s, %s\n", PrfVersion, __DATE__, __TIME__);
  printf("1-dim. boundary layer models\n");
  printf("Copyright (C) Janicke Consulting, Ueberlingen, Germany, 2000-2010\n");
  printf("IBJprf comes with ABSOLUTELY NO WARRANTY\n");
  printf("IBJprf is free software under the GNU PUBLIC LICENCE\n\n");
  Lm = 0.;
  Hm = 0.;
  Ua = 0.;
  Ha = 0.;
  Ust = 0.;
  Nz = 0.;
  Z0 = 0.;
  D0 = 0.;
  Ra = 0.;
  Kappa = PRF_KAPPA;
  Interval = 3600.;
  sprintf(Name, "");
  if (argc < 2) Help();
  for (i=1; i<argc; i++) {
    pc = argv[i];
    if (*pc == '-') {
      switch (pc[1]) {
        case 'l': j=sscanf(pc+2, "%f", &Lm);
                  if (j != 1) Help();
                  break;
        case 'm': j=sscanf(pc+2, "%f", &Hm);
                  if (j != 1) Help();
                  break;
        case 'u': j=sscanf(pc+2, "%f", &Ua);
                  if (j != 1) Help();
                  break;
        case 'h': j=sscanf(pc+2, "%f", &Ha);
                  if (j != 1) Help();
                  break;
        case 'i': j=sscanf(pc+2, "%f", &Interval);
                  if (j != 1) Help();
                  break;
        case 'd': j=sscanf(pc+2, "%f", &D0);
                  if (j != 1) Help();
                  break;
        case 'k': j=sscanf(pc+2, "%f", &Kappa);
                  if (j != 1) Help();
                  break;
        case 's': j=sscanf(pc+2, "%f", &Ust);
                  if (j != 1) Help();
                  break;
        case 'r': j=sscanf(pc+2, "%f", &Ra);
                  if (j != 1) Help();
                  break;
        case 'z': j=sscanf(pc+2, "%f", &Z0);
                  if (j != 1) Help();
                  break;
        case 'f': j=sscanf(pc+2, "%s", &Name);
                  if (j != 1) Help();
                  break;
        case 'V': j=sscanf(pc+2, "%f", &z);
                  if (j != 1) Help();
                  Version = (int)(z);
                  PrfSetVersion(Version);
                  break;
        case 'v': j = 0;
                  pc += 2;
                  while (1 == sscanf(pc, "%f", &z)) {
                    while ((*pc) && *pc != ',') pc++;
                    if (!(*pc)) break;
                    pc++;
                    Zz[j++] = z;
                  }
                  Nz = j;
                  break;
      }
    }
    else
      Help();
  }
  profiles();
  printf("Program IBJprf finished.\n");
  return 0;
}

#endif  /*###################################################################*/
